# 종목 분석용 주역 기반 진단 Python 자동화 도구 (지표 자동 수집 포함)
# 웹 기반 입력 → 주가 지표 분석(yfinance 기반) → 괘 매핑 → 전략 제시
import yfinance as yf
from flask import Flask, request, render_template_string
import pandas as pd

app = Flask(__name__)

# 주역 괘 정보 매핑 테이블
hexagram_mapping = {
    "강한 상승": {
        "괘": "乾(건)",
        "의미": "하늘이 끊임없이 움직이듯 상승 에너지가 강함",
        "전략": "상승 추세 지속. 모멘텀 전략 유효. 수익 실현 시점 검토."
    },
    "완만한 상승": {
        "괘": "兌(태)",
        "의미": "기쁨 속에서도 절제된 상승. 우호적이지만 경계는 필요함.",
        "전략": "추세 추종 가능. 조정 시 분할 매수."
    },
    "조정": {
        "괘": "坤(곤)",
        "의미": "부드럽게 수용하는 땅. 큰 방향은 유지하되 일시 조정.",
        "전략": "추가 하락 가능성 대비. 펀더멘털 검토 병행."
    },
    "과열": {
        "괘": "火(화)",
        "의미": "강한 빛은 오래가지 않음. 과열 상태 경고.",
        "전략": "단기 고점 가능성. 리스크 관리 및 이익 실현 고려."
    },
    "약세": {
        "괘": "水(수)",
        "의미": "흐름이 아래로 향함. 전환 신호가 없으면 하락 지속.",
        "전략": "관망 또는 단기 매도 전략. 저점 탐색 필요."
    }
}

# 주요 지표 계산 함수
# 입력: yfinance에서 수집된 주가 데이터 (DataFrame)
def calculate_indicators(df):
    df["Return"] = df["Close"].pct_change(periods=20) * 100  # 최근 1개월 수익률
    df["MA20"] = df["Close"].rolling(window=20).mean()
    df["MA60"] = df["Close"].rolling(window=60).mean()
    df["Volatility"] = df["Close"].rolling(window=20).std()

    # RSI 계산
    delta = df["Close"].diff()
    gain = delta.where(delta > 0, 0)
    loss = -delta.where(delta < 0, 0)
    avg_gain = gain.rolling(window=14).mean()
    avg_loss = loss.rolling(window=14).mean()
    rs = avg_gain / avg_loss
    df["RSI"] = 100 - (100 / (1 + rs))

    return df

# 상태 판단 기준 적용 함수
def interpret_state(df):
    latest = df.iloc[-1]
    return_val = latest["Return"]
    rsi = latest["RSI"]
    volatility = latest["Volatility"]

    if return_val > 10 and rsi > 70:
        signal = "과열"
    elif return_val > 5:
        signal = "강한 상승"
    elif return_val > 0:
        signal = "완만한 상승"
    elif return_val > -5:
        signal = "조정"
    else:
        signal = "약세"

    return {
        "signal": signal,
        "return": return_val,
        "rsi": rsi,
        "volatility": volatility,
        "hexagram": hexagram_mapping[signal]
    }

# 종목 분석 핵심 함수
def analyze_stock(ticker):
    try:
        stock = yf.Ticker(ticker)
        df = stock.history(period="3mo")
        df = calculate_indicators(df)
        result = interpret_state(df)
        result["ticker"] = ticker
        return result
    except Exception as e:
        return {"error": str(e), "ticker": ticker}

# 웹 인터페이스 템플릿
html_template = """
<!doctype html>
<html lang="ko">
  <head><meta charset="utf-8"><title>주역 기반 종목 진단기</title></head>
  <body>
    <h2>주역 기반 종목 진단기</h2>
    <form method="POST">
      종목 티커 입력 (예: AAPL): <input type="text" name="ticker">
      <input type="submit" value="진단하기">
    </form>
    {% if result %}
      {% if result.error %}
        <p style="color:red;">에러: {{ result.error }}</p>
      {% else %}
        <h3>분석 결과 ({{ result.ticker }})</h3>
        <p>▶ 상태 진단: <b>{{ result.signal }}</b></p>
        <p>▶ 괘: {{ result.hexagram.괘 }} – {{ result.hexagram.의미 }}</p>
        <p>▶ 전략 제시: {{ result.hexagram.전략 }}</p>
        <hr>
        <p>추정 지표:</p>
        <ul>
          <li>1개월 수익률: {{ result.return | round(2) }}%</li>
          <li>RSI(14): {{ result.rsi | round(2) }}</li>
          <li>변동성(20일 표준편차): {{ result.volatility | round(2) }}</li>
        </ul>
      {% endif %}
    {% endif %}
  </body>
</html>
"""

@app.route("/", methods=["GET", "POST"])
def index():
    result = None
    if request.method == "POST":
        ticker = request.form["ticker"].upper()
        result = analyze_stock(ticker)
    return render_template_string(html_template, result=result)

# 실행용
if __name__ == "__main__":
    app.run(debug=True)
